# flake8: noqa: E501
class Const(object):
    def __init__(self):
        self.header = """<?xml version="1.0" encoding="utf-8"?>\n<gpx creator="jujumini" version="1.1" xsi:schemaLocation="http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd http://www.garmin.com/xmlschemas/WaypointExtension/v1 http://www8.garmin.com/xmlschemas/WaypointExtensionv1.xsd http://www.garmin.com/xmlschemas/TrackPointExtension/v1 http://www.garmin.com/xmlschemas/TrackPointExtensionv1.xsd http://www.garmin.com/xmlschemas/GpxExtensions/v3 http://www8.garmin.com/xmlschemas/GpxExtensionsv3.xsd http://www.garmin.com/xmlschemas/ActivityExtension/v1 http://www8.garmin.com/xmlschemas/ActivityExtensionv1.xsd http://www.garmin.com/xmlschemas/AdventuresExtensions/v1 http://www8.garmin.com/xmlschemas/AdventuresExtensionv1.xsd http://www.garmin.com/xmlschemas/PressureExtension/v1 http://www.garmin.com/xmlschemas/PressureExtensionv1.xsd http://www.garmin.com/xmlschemas/TripExtensions/v1 http://www.garmin.com/xmlschemas/TripExtensionsv1.xsd http://www.garmin.com/xmlschemas/TripMetaDataExtensions/v1 http://www.garmin.com/xmlschemas/TripMetaDataExtensionsv1.xsd http://www.garmin.com/xmlschemas/ViaPointTransportationModeExtensions/v1 http://www.garmin.com/xmlschemas/ViaPointTransportationModeExtensionsv1.xsd http://www.garmin.com/xmlschemas/CreationTimeExtension/v1 http://www.garmin.com/xmlschemas/CreationTimeExtensionsv1.xsd http://www.garmin.com/xmlschemas/AccelerationExtension/v1 http://www.garmin.com/xmlschemas/AccelerationExtensionv1.xsd http://www.garmin.com/xmlschemas/PowerExtension/v1 http://www.garmin.com/xmlschemas/PowerExtensionv1.xsd http://www.garmin.com/xmlschemas/VideoExtension/v1 http://www.garmin.com/xmlschemas/VideoExtensionv1.xsd" xmlns="http://www.topografix.com/GPX/1/1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:wptx1="http://www.garmin.com/xmlschemas/WaypointExtension/v1" xmlns:gpxtrx="http://www.garmin.com/xmlschemas/GpxExtensions/v3" xmlns:gpxtpx="http://www.garmin.com/xmlschemas/TrackPointExtension/v1" xmlns:gpxx="http://www.garmin.com/xmlschemas/GpxExtensions/v3" xmlns:trp="http://www.garmin.com/xmlschemas/TripExtensions/v1" xmlns:adv="http://www.garmin.com/xmlschemas/AdventuresExtensions/v1" xmlns:prs="http://www.garmin.com/xmlschemas/PressureExtension/v1" xmlns:tmd="http://www.garmin.com/xmlschemas/TripMetaDataExtensions/v1" xmlns:vptm="http://www.garmin.com/xmlschemas/ViaPointTransportationModeExtensions/v1" xmlns:ctx="http://www.garmin.com/xmlschemas/CreationTimeExtension/v1" xmlns:gpxacc="http://www.garmin.com/xmlschemas/AccelerationExtension/v1" xmlns:gpxpx="http://www.garmin.com/xmlschemas/PowerExtension/v1" xmlns:vidx1="http://www.garmin.com/xmlschemas/VideoExtension/v1">\n\n"""
        self.metadata = """<metadata>
        <link href="http://www.garmin.com">
          <text>Garmin International</text>
        </link>
        <time>{time}</time>
      </metadata>\n\n"""
        self.waypoint = """  <wpt lat = "{lat}" lon = "{lon}">\n    <name>{name}</name>\n    <desc>{desc}</desc>\n    <sym>{mark}, {color}</sym>\n    <type>user</type>\n  </wpt>\n\n"""
        self.route = """    <rtept lat="{lat}" lon="{lon}">\n      <name>{name}</name>\n      <desc>{desc}</desc>\n      <sym>{mark}, {color}</sym>\n    </rtept>\n\n"""
        self.routeStart = """  <rte>\n    <name>{name}</name>\n"""
        self.routeEnd = """  </rte>\n\n"""
        self.footer = "</gpx>"
        self.colors = ["Black", "Red", "Yellow", "Green", "Brown", "Purple", "Blue"]
        self.marks = {
            "@q": "Pin",
            "@r": "Flag",
            "@s": "Flag",
            "@t": "Fishing Area",
            "@u": "Fishing Area",
            "@v": "Fishing Area",
            "@w": "Shipwreck",
            "@x": "Anchor",
            "@y": "Skull And Crossbones",
            "@z": "Flag",
        }
        self.defaultMark = "@q"
        self.defaultColor = 0
        self.defaultColorText = "Blue"



from click import Choice
import typer
import os
import time
from enum import Enum
import click
import gp32_transfer
from .utils.serial_util import save_gps_to_gpx
from typing import List, Optional
from .utils.serial_util import from_gpx_to_gps
from pathlib import Path

from gp32_transfer.utils import serial_util

help = "App used to import or export gpx with a Furuno GP32 gps"
app = typer.Typer(help=help)
state = {"port": ""}


@app.command("import")
def import_from_gps(
    filename: str = typer.Argument(..., help="Specify filename for gpx export"),
    output_dir: Optional[str] = typer.Argument(None, help="Specify output directory"),
    nmea: Optional[bool] = typer.Option(False, help="Get raw NMEA file"),
    csv: Optional[bool] = typer.Option(False, help="Get csv of waypoints"),
):
    """
    Import gps waypoints from GP32 to gpx
    """
    if output_dir:
        filename = os.path.join(output_dir, filename)
    else:
        output_dir = "here"
    typer.secho(f"File(s) will be exported to {output_dir}", fg=typer.colors.MAGENTA)
    time.sleep(1)

    typer.secho("\n" + "-" * 30 + "\n")

    typer.secho("Press 'Sauve WP/RTE -> PC?'", fg=typer.colors.BRIGHT_YELLOW)
    typer.secho("and press 'Poursuivre'", fg=typer.colors.BRIGHT_YELLOW)

    typer.secho("\n" + "-" * 30 + "\n")
    save_gps_to_gpx(port=state["port"],filename=filename, nmea_flag=nmea, csv_flag=csv)


@app.command("export")
def export_to_gps(gpx_file: Path = typer.Argument(..., help="gpx file to upload")):
    """
    Export gps waypoints from gpx to GP32
    """
    if gpx_file is None:
        typer.echo("No file entered")
        raise typer.Abort()
    if gpx_file.is_file():
        typer.echo("\n" + "-" * 30)
        typer.echo(f"File {gpx_file} will be processed")
    elif not gpx_file.exists():
        typer.echo("The file doesn't exist")
        raise typer.Abort()

    typer.echo("\n" + "-" * 30)
    typer.echo("Connection to GP32 is good")
    typer.echo("Press 'Charge WP/RTE <- PC?'")
    typer.echo("and press 'Poursuivre'")

    typer.echo("\n" + "-" * 30)
    typer.secho(f"Have you completed the steps before? ", fg=typer.colors.BRIGHT_YELLOW)
    flag_start = typer.confirm("Is GP32 ready to receive data ?", default=True)
    if flag_start:
        from_gpx_to_gps(gpx_file=gpx_file,port=state["port"])

    typer.echo("\n" + "-" * 30)
    typer.echo("End of program")
    raise typer.Exit()


@app.callback()
def set_serial_port():
    ports_list = serial_util.searchcom()
    typer.echo("\n" + "-" * 30)
    port_chosen = click.prompt(
        "Select a port to open",
        show_default=True,
        type=click.Choice([x for x in ports_list]),
        default=ports_list[0],
    )
    state["port"] = port_chosen
    typer.echo("\n" + "-" * 30)
    typer.echo("You are ready to import/export on the GPS")


# def main():
#     typer.echo("Hello World")


if __name__ == "__main__":
    app()




import re
import math
from xml.etree import cElementTree
import serial

from . import const

# Const.py

Const = const.Const()


def parse_nmea_line(line):
    break_code = False
    wpt = {}

    msg = re.sub(r'[\n\r]', '', line)
    splitstr = re.split(',', msg)

    # parse waypoints & routes
    if (splitstr and len(splitstr) > 1):
        if splitstr[0] == '$PFEC' and splitstr[1] == 'GPwpl':
            # find description/mark
            mark = re.match('^(\@[a-z]{1})(.+)$', splitstr[8])
            wpt = {
                'name': splitstr[6].strip(),
                'color': Const.colors[int(splitstr[7])] if splitstr[7] and len(splitstr[7]) > 0 and int(
                    splitstr[7]) < len(
                    Const.colors) else Const.defaultColorText,
                'mark': Const.defaultMark,
                'desc': mark.group(2).strip() if mark and mark.group(2).strip() else splitstr[8][2:].strip(),
                'lat': round(float(splitstr[2][2:]) / 60 + float(splitstr[2][:2]), 15) if splitstr[
                    3] == 'N' else round(
                    -float(splitstr[2][2:]) / 60 - float(splitstr[2][:2]), 15),
                'lon': round(float(splitstr[4][3:]) / 60 + float(splitstr[4][:3]), 15) if splitstr[
                    5] == 'E' else round(
                    -float(splitstr[4][3:]) / 60 - float(splitstr[4][:3]), 15)
            }
    return wpt



import serial
import os
from . import furuno_read
from . import furuno_write
import re
import pandas as pd
import time
from datetime import datetime
from . import const


def write_raw_nmea(file_name, data):
    _time = str(datetime.strftime(datetime.now(), "%Y-%m-%dT%H:%M:%SZ"))
    with open(f"{file_name}_{_time}.nmea", "w") as outfile:
        outfile.write("\n".join(data))
    print(f"Raw nmea saved as {file_name}_{_time}.nmea")


def write_raw_gpx(file_name, data):
    _time = str(datetime.strftime(datetime.now(), "%Y-%m-%dT%H:%M:%SZ"))
    with open(f"{file_name}_{_time}.gpx", "w") as outfile:
        outfile.write("\n".join(data))


def write_parsed_nmea(file_name, data):
    _time = str(datetime.strftime(datetime.now(), "%Y-%m-%dT%H:%M:%SZ"))

    df = pd.DataFrame.from_dict(data)
    df.to_csv(f"{file_name}_{_time}.csv", index=False)


def searchcom():
    # chose an implementation, depending on os
    # ~ if sys.platform == 'cli':
    # ~ else:
    if os.name == "nt":  # sys.platform == 'win32':
        from serial.tools.list_ports_windows import comports
    elif os.name == "posix":
        from serial.tools.list_ports_posix import comports
    # ~ elif os.name == 'java':
    else:
        raise ImportError(
            "Sorry: no implementation for COM ports \
                for your platform ('{}') available".format(
                os.name
            )
        )
    iterator = sorted(comports())
    return [data[0].strip() for data in iterator]


def save_raw_nmea(filename, nmea_flag=False, port="/dev/ttyUSB0"):
    com_port = port
    baud_rate = 4800
    ser = serial.Serial(
        com_port,
        baud_rate,
        parity=serial.PARITY_NONE,
        stopbits=serial.STOPBITS_ONE,
        bytesize=serial.EIGHTBITS,
        xonxoff=True,
    )

    if ser.is_open:
        ser.close()
        time.sleep(2)

    raw_nmea = []

    ser.open()
    while True:
        line = str(ser.readline().decode("utf-8").strip())
        if "PFEC" in line:
            raw_nmea.append(line)
            if "GPxfr" not in line:
                print("Received waypoint: " + re.split(",", line)[6])

        if "GPxfr" in line:
            ser.close()
            break

    print(f"{len(raw_nmea)} waypoints exported.")
    ser.close()

    if nmea_flag:
        write_raw_nmea(file_name=filename, data=raw_nmea)
    return raw_nmea


def save_all_waypoints(
    filename,
    port,
    nmea_flag=False,
    csv_flag=False,
    debug=False,
):
    if debug:
        with open("raw_nmea.nmea", "r") as nmea:
            rwa_nmea = nmea.readlines()
            rwa_nmea = list(map(str.strip, rwa_nmea))
    else:
        rwa_nmea = save_raw_nmea(filename=filename, port=port, nmea_flag=nmea_flag)

    list_wpts = []

    for line in rwa_nmea:
        wpt = furuno_read.parse_nmea_line(line)
        if wpt:
            list_wpts.append(wpt)

    if csv_flag:
        write_parsed_nmea(filename, list_wpts)

    return list_wpts


def write_to_gpx(list_wpts, file_name="test_gpx", debug=False):
    Const = const.Const()
    if debug:
        save_all_waypoints(filename=file_name, debug=True)
    gpx_wpt = []
    gpx_wpt.append(Const.header)
    gpx_wpt.append(
        Const.metadata.format(
            time=str(datetime.strftime(datetime.now(), "%Y-%m-%dT%H:%M:%SZ"))
        )
    )

    for v in list_wpts:
        gpx_wpt.append(
            Const.waypoint.format(
                desc=v["desc"],
                mark=v["mark"],
                color=v["color"],
                name=v["name"],
                lat=v["lat"],
                lon=v["lon"],
            )
        )
    gpx_wpt.append(Const.footer)
    write_raw_gpx(file_name, gpx_wpt)


def save_gps_to_gpx(port, filename="GP32_test.gpx", nmea_flag=False, csv_flag=False):
    wpts = save_all_waypoints(
        filename=filename, nmea_flag=nmea_flag, csv_flag=csv_flag, port=port
    )
    write_to_gpx(wpts, filename)


def write_to_gps(list_nmea, port="/dev/ttyUSB0"):
    com_port = port
    baud_rate = 4800
    ser = serial.Serial(
        com_port,
        baud_rate,
        parity=serial.PARITY_NONE,
        stopbits=serial.STOPBITS_ONE,
        bytesize=serial.EIGHTBITS,
        xonxoff=True,
    )

    if ser.is_open:
        ser.close()
        time.sleep(2)

    ser.open()

    for i in range(len(list_nmea)):
        print(f'{format((i+1) / len(list_nmea) *100, ".2f")} % completed')
        ser.write(list_nmea[i].encode("utf-8", "replace") + b"\r\n")

    ser.close()


def from_gpx_to_gps(gpx_file, port):
    listnmea = furuno_write.get_nmea_from_file(gpx_file)
    write_to_gps(listnmea, port)


if __name__ == "__main__":
    # save_raw_nmea()

    Const = const.Const()
    list_nmea = furuno_write.get_nmea_from_file(gpx_file="test2.gpx")
    print(list_nmea)



from xml.etree import cElementTree
import re
import math
from . import const

# Const.py

Const = const.Const()


def read_file(path_of_file):
    try:
        root = cElementTree.parse(path_of_file).getroot()
    except:
        print("Could not open input file!")
    return root


def extract_wpt(node):
    # find waypoint
    lon = float(node.attrib["lon"])
    furuno_lon = round(
        math.floor(abs(lon)) * 100 + ((abs(lon) - math.floor(abs(lon))) * 60), 3
    )
    lat = float(node.attrib["lat"])
    furuno_lat = round(
        math.floor(abs(lat)) * 100 + ((abs(lat) - math.floor(abs(lat))) * 60), 3
    )
    tmp_wpt = {
        "lon": "%09.3f" % furuno_lon,
        "NS": "S" if lon > 0 else "N",
        "lat": "%08.3f" % furuno_lat,
        "EW": "W" if lat > 0 else "E",
        "name": "",
        "desc": "",
        "color": Const.defaultColor,
        "mark": Const.defaultMark,
    }
    for wpt in node:
        if re.search("\}name", wpt.tag) and wpt.text:
            tmp_wpt["name"] = wpt.text.strip().upper()
        if re.search("\}desc", wpt.tag) and wpt.text:
            tmp_wpt["desc"] = wpt.text.strip().upper()
        if re.search("\}sym", wpt.tag) and wpt.text:
            splitSym = re.split(",", wpt.text)
            if len(splitSym) == 2:
                foundmark = [
                    f for f in Const.marks if (Const.marks[f] == splitSym[0].strip())
                ]
                tmp_wpt["mark"] = (
                    foundmark[0] if len(foundmark) > 0 else Const.defaultMark
                )
                tmp_wpt["color"] = (
                    Const.colors.index(splitSym[1].strip())
                    if splitSym[1].strip() in Const.colors
                    else Const.defaultColor
                )
            else:
                foundmark = [
                    f for f in Const.marks if (Const.marks[f] == wpt.text.strip())
                ]
                tmp_wpt["mark"] = (
                    foundmark[0] if len(foundmark) > 0 else Const.defaultMark
                )
    return tmp_wpt


def extract_route(node):
    tmpRoute = {"name": "", "desc": "", "track": []}
    for rte in node:
        if re.search("\}name$", rte.tag) and rte.text:
            tmpRoute["name"] = rte.text.strip().upper()
            # take only 8 first symbols in route name  for furuno 32 version compatible!
            tmpRoute["name"] = (
                tmpRoute["name"] if len(tmpRoute["name"]) <= 8 else tmpRoute["name"][:8]
            )

        if re.search("\}desc$", rte.tag) and rte.text:
            tmpRoute["desc"] = rte.text.strip().upper()
        if (
            re.search("\}rtept$", rte.tag)
            and ("lon" in rte.attrib)
            and ("lat" in rte.attrib)
        ):
            lon = float(rte.attrib["lon"])
            furuno_lon = round(
                math.floor(abs(lon)) * 100 + ((abs(lon) - math.floor(abs(lon))) * 60), 3
            )
            lat = float(rte.attrib["lat"])
            furuno_lat = round(
                math.floor(abs(lat)) * 100 + ((abs(lat) - math.floor(abs(lat))) * 60), 3
            )
            tmpRoute["track"].append(
                {
                    "lon": "%09.3f" % furuno_lon,
                    "NS": "S" if lon > 0 else "N",
                    "lat": "%08.3f" % furuno_lat,
                    "EW": "W" if lat > 0 else "E",
                    "name": "",
                    "desc": "",
                    "color": Const.defaultColor,
                    "mark": Const.defaultMark,
                }
            )
            routesTrackCounter = len(tmpRoute["track"]) - 1
            for rtept in rte:
                if re.search("\}name", rtept.tag) and rtept.text:
                    tmpRoute["track"][routesTrackCounter][
                        "name"
                    ] = rtept.text.strip().upper()
                if re.search("\}desc", rtept.tag) and rtept.text:
                    tmpRoute["track"][routesTrackCounter][
                        "desc"
                    ] = rtept.text.strip().upper()
                if re.search("\}sym", rtept.tag) and rtept.text:
                    splitSym = re.split(",", rtept.text)
                    if len(splitSym) == 2:
                        foundmark = [
                            f
                            for f in Const.marks
                            if (Const.marks[f] == splitSym[0].strip())
                        ]
                        tmpRoute["track"][routesTrackCounter]["mark"] = (
                            foundmark[0] if len(foundmark) > 0 else Const.defaultMark
                        )
                        tmpRoute["track"][routesTrackCounter]["color"] = (
                            Const.colors.index(splitSym[1].strip())
                            if splitSym[1].strip() in Const.colors
                            else Const.defaultColor
                        )
                    else:
                        foundmark = [
                            f
                            for f in Const.marks
                            if (Const.marks[f] == rtept.text.strip())
                        ]
                        tmpRoute["track"][routesTrackCounter]["mark"] = (
                            foundmark[0] if len(foundmark) > 0 else Const.defaultMark
                        )
    return tmpRoute


def parse_gpx(gpx):
    try:
        parseWaypoints = {}
        parseRoutes = {}
        waypointsCounter = 0
        routesCounter = 0

        list_name_longer_than_6 = []
        list_wpt_in_double = []

        for child in gpx:
            if (
                re.search("\}wpt$", child.tag)
                and ("lon" in child.attrib)
                and ("lat" in child.attrib)
            ):
                # extract_wpt(child)
                wpt = extract_wpt(child)

                if (
                    wpt["name"] in parseWaypoints.keys()
                    and wpt["name"] not in list_wpt_in_double
                ):
                    list_wpt_in_double.append(wpt["name"])

                if len(wpt["name"]) > 6:
                    list_name_longer_than_6.append(wpt["name"])

                parseWaypoints[wpt["name"]] = wpt
                waypointsCounter += 1

            if re.search("\}rte$", child.tag):
                parseRoutes[routesCounter] = extract_route(child)
                routesCounter += 1

        error = [list_name_longer_than_6, list_wpt_in_double]

        return (parseWaypoints, parseRoutes, error)
    except:
        print("Error parsing GPX")


def get_nmea_from_parsed_gpx(wpt_parsed, rte_parsed):
    putData = []
    for _, value in wpt_parsed.items():

        lat = value["lat"]
        ns = value["NS"]
        lon = value["lon"]
        ew = value["EW"]
        name = value["name"]
        color = value["color"]
        desc = str(value["mark"]) + str(value["desc"])

        putData.append(
            f"$PFEC,GPwpl,{lat},{ns},{lon},{ew},{name:6.6},{color},{desc:18.18},A,,,,"
        )

    route_num = 0
    for _, value in rte_parsed.items():
        # print(value)
        if value["track"] and len(value["track"]) > 0:
            route_num += 1
            sentence_num = math.ceil(len(value["track"]) / 8)
            sentence_count = 1
            inner_sentence_count = 1
            line = f"$GPRTE,{sentence_num},{sentence_count},C,{route_num:02d}"

            for x in range(0, len(value["track"])):
                name = value["track"][x]["name"]
                line += f", {name:6}"
                if inner_sentence_count == 8 or x == len(value["track"]) - 1:
                    inner_sentence_count = 1
                    sentence_count += 1
                    putData.append(line)
                    line = f"$GPRTE,{sentence_num},{sentence_count},C,{route_num:02d}"
                else:
                    inner_sentence_count += 1

            # name track
            route_num = route_num
            name = value["name"]
            putData.append(f"$PFEC,GPrtc,{route_num:02d},{name:16.16}")

    putData.append("$PFEC,GPxfr,CTL,E")
    return putData


def get_nmea_from_file(gpx_file):
    gpx = read_file(gpx_file)
    wpts, rtes, error = parse_gpx(gpx)
    if len(error[0]) > 0 or len(error[1]) > 0:
        print("error in parsing")
    print(f"\nThere are {len(wpts)} waypoints and {len(rtes)} routes to export")
    return get_nmea_from_parsed_gpx(wpts, rtes)


if __name__ == "__main__":
    test_file = "test2.gpx"
    get_nmea_from_file(test_file)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gp32_transfer', 'gp32_transfer.utils']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.3.5,<2.0.0', 'pyserial==3.5', 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['gp32_transfer = gp32_transfer.main:app']}

setup_kwargs = {
    'name': 'gp32-transfer',
    'version': '0.2.2',
    'description': '',
    'long_description': '<div id="top"></div>\n<!--\n*** Thanks for checking out the Best-README-Template. If you have a suggestion\n*** that would make this better, please fork the repo and create a pull request\n*** or simply open an issue with the tag "enhancement".\n*** Don\'t forget to give the project a star!\n*** Thanks again! Now go create something AMAZING! :D\n-->\n\n<!-- PROJECT SHIELDS -->\n<!--\n*** I\'m using markdown "reference style" links for readability.\n*** Reference links are enclosed in brackets [ ] instead of parentheses ( ).\n*** See the bottom of this document for the declaration of the reference variables\n*** for contributors-url, forks-url, etc. This is an optional, concise syntax you may use.\n*** https://www.markdownguide.org/basic-syntax/#reference-style-links\n-->\n<!--\n[![Contributors][contributors-shield]][contributors-url]\n[![Forks][forks-shield]][forks-url]\n[![Stargazers][stars-shield]][stars-url]\n[![Issues][issues-shield]][issues-url]\n[![MIT License][license-shield]][license-url]\n[![LinkedIn][linkedin-shield]][linkedin-url] -->\n\n<!-- PROJECT LOGO -->\n<br />\n<div align="center">\n\n<h3 align="center">GP32 GPS Communication 3000</h3>\n<!-- \n  <p align="center">\n    Interface which can save and upload waypoint to Furuno GP32 using RS232\n    <br />\n    <a href="https://github.com/pepin_nucleaire/furuno-gpx-transfer"><strong>Explore the docs »</strong></a>\n    <br />\n    <br />\n    <a href="https://github.com/pepin_nucleaire/furuno-gpx-transfer">View Demo</a>\n    ·\n    <a href="https://github.com/pepin_nucleaire/furuno-gpx-transfer/issues">Report Bug</a>\n    ·\n    <a href="https://github.com/pepin_nucleaire/furuno-gpx-transfer/issues">Request Feature</a>\n  </p>-->\n</div>\n\n<!-- TABLE OF CONTENTS -->\n<!-- <details>\n  <summary>Table of Contents</summary>\n  <ol>\n    <li>\n      <a href="#about-the-project">About The Project</a>\n      <ul>\n        <li><a href="#built-with">Built With</a></li>\n      </ul>\n    </li>\n    <li>\n      <a href="#getting-started">Getting Started</a>\n      <ul>\n        <li><a href="#prerequisites">Prerequisites</a></li>\n        <li><a href="#installation">Installation</a></li>\n      </ul>\n    </li>\n    <li><a href="#usage">Usage</a></li>\n    <li><a href="#roadmap">Roadmap</a></li>\n    <li><a href="#contributing">Contributing</a></li>\n    <li><a href="#license">License</a></li>\n    <li><a href="#contact">Contact</a></li>\n    <li><a href="#acknowledgments">Acknowledgments</a></li>\n  </ol>\n</details> -->\n\n<!-- GETTING STARTED -->\n\n## Getting Started\n\nThis is an example of how you may give instructions on setting up your project locally.\nTo get a local copy up and running follow these simple example steps.\n\n### Installation\n\n<!-- USAGE EXAMPLES -->\n\n## Usage\n\n<!-- ROADMAP -->\n\n## Roadmap\n\n- [x] ~~Save waypoints~~\n  - [x] ~~Read from GP32~~\n  - [x] ~~Save to GPX file compatible with OpenCPN~~\n- [] Upload Waypoints\n  - [x] ~~Convert GPX to NMEA~~\n  - [x] ~~Upload NMEA to GP32~~\n- [x] Make a nice app\n  - [x] CLI\n  - [] Web-based ?\n  - [] Click-n-go ?\n\nSee the [open issues](https://github.com/pepin_nucleaire/furuno-gpx-transfer/issues) for a full list of proposed features (and known issues).\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- CONTRIBUTING -->\n\n## Contributing\n\nContributions are what make the open source community such an amazing place to learn, inspire, and create. Any contributions you make are **greatly appreciated**.\n\nIf you have a suggestion that would make this better, please fork the repo and create a pull request. You can also simply open an issue with the tag "enhancement".\nDon\'t forget to give the project a star! Thanks again!\n\n1. Fork the Project\n2. Create your Feature Branch (`git checkout -b feature/AmazingFeature`)\n3. Commit your Changes (`git commit -m \'Add some AmazingFeature\'`)\n4. Push to the Branch (`git push origin feature/AmazingFeature`)\n5. Open a Pull Request\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- LICENSE -->\n\n## License\n\nDistributed under the MIT License. See `LICENSE.txt` for more information.\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- CONTACT -->\n\n## Contact\n\nYour Name - [@juju_on_mini](https://twitter.com/juju_on_mini) - muller.julien.02@gmail.com.com\n\nProject Link: [https://github.com/pepin_nucleaire/furuno-gpx-transfer](https://github.com/pepin_nucleaire/furuno-gpx-transfer)\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- ACKNOWLEDGMENTS -->\n\n## Acknowledgments\n\n- [russkiy78 and it furunotogpx project](https://github.com/russkiy78/furunotogpx) that I used as a really big inspiration\n- My dog\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- MARKDOWN LINKS & IMAGES -->\n<!-- https://www.markdownguide.org/basic-syntax/#reference-style-links -->\n',
    'author': 'Julien Muller',
    'author_email': 'muller.julien.02@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
